/*
  Copyright (C) 2004-2005 Tommi Tervonen, Petteri Klemola, Pasi Orovuo, Marko Mattila

  This file is part of Kajaani Kombat.

  Kajaani Kombat is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  Kajaani Kombat is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with Kajaani Kombat; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "client_net.h"

client_tcpnet::client_tcpnet(IPaddress *server_ip) throw(string &)
{
  socket = SDLNet_TCP_Open (server_ip);
  if (socket == NULL)
    throw string ("Error connecting to server: ") + SDLNet_GetError();
  socketset = SDLNet_AllocSocketSet(1);

  int ret = SDLNet_TCP_AddSocket(socketset, socket);
  assert (ret != -1);
}

client_tcpnet::~client_tcpnet()
{
  SDLNet_TCP_Close(socket);
  SDLNet_TCP_DelSocket(socketset, socket);
  SDLNet_FreeSocketSet (socketset);
}

msg * client_tcpnet::recv() throw (string &)
{
  msg *m = msg::recv(socket);
  return m;
}

void client_tcpnet::send(msg *m) throw (string &)
{
  m->send(socket);
}

bool client_tcpnet::has_msgs() throw (string &)
{
  return SDLNet_CheckSockets (socketset, 0) > 0;
}


msg * client_localnet::recv() throw (string &)
{
  return in.pop();
}

void client_localnet::send(msg * m) throw (string &)
{
  out.push(m->deep_copy());
}

bool client_localnet::has_msgs() throw (string &)
{
  return !in.empty();
}

monitor_queue<msg *> * client_localnet::get_in_queue()
{
  return &in;
}

monitor_queue<msg *> * client_localnet::get_out_queue()
{
  return &out;
}

client_localnet::client_localnet()
{

}

client_localnet::~client_localnet()
{

}
