/*
  Copyright (C) 2004-2005 Tommi Tervonen, Petteri Klemola, Pasi Orovuo

  This file is part of Kajaani Kombat.

  Kajaani Kombat is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  Kajaani Kombat is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with Kajaani Kombat; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "gui_screens.h"

typedef struct server_numpl {
  game_server *server;
  int num_players;
};

const char * main_screen::button_files[4][3] = {
  { "main_butstart.png", "main_butstart_focus.png", "main_butstart_act.png" },
  { "main_buthost.png", "main_buthost_focus.png", "main_buthost_act.png" },
  { "main_butopt.png", "main_butopt_focus.png", "main_butopt_act.png" },
  { "main_butquit.png", "main_butquit_focus.png", "main_butquit_act.png" }
};

const char * numpl_screen::button_files[4][3] = {
  { "numpl_butone.png", "numpl_butone_focus.png", "numpl_butone_act.png" },
  { "numpl_buttwo.png", "numpl_buttwo_focus.png", "numpl_buttwo_act.png" }
};

const char *numpl_screen::imgtext_files[1] = {
  "numpl_imgtext.png"
};

const char *connect_screen::imgtext_files[4] = {
  "connect_imgname.png", "connect_imgname2.png", "connect_imgserv.png", "connect_imgport.png"
};

const char *host_screen::imgtext_files[6] = {
  "host_imgname.png", "host_imgname2.png", "host_imgport.png", "host_imgnumpl.png", "host_imgnumcastles.png",
  "host_imgnumwins.png"
};

const char *host_screen::button_files[][3] = {
  { "host_butstart.png", "host_butstart_focus.png", "host_butstart_act.png" }
};

const char *connect_screen::button_files[][3] = {
  { "connect_butconn.png", "connect_butconn_focus.png", "connect_butconn_act.png" }
};

main_screen::main_screen(SDL_Surface *scr, TTF_Font *font, gfx *graphix, starfield_efect *se, audio *au)
  : gui_screen (scr, font, graphix, se, au)
{
  for(int p=0;p<4;p++)
    {
      for (int i=0;i<3;i++)
	{
	  //	  printf ("Loading button image %s\n", button_files[p][i]);
	  button_gfx[p][i] = img_load (button_files[p][i]);
	  if (button_gfx[p][i] == NULL)
	    {
	      fprintf (stderr, "Error loading image %s: %s\n", button_files[p][i], IMG_GetError());
	      exit (1);
	    }
	}
    }
  logogfx = img_load("logo.png");
  if (logogfx == NULL)
    {
      fprintf (stderr, "Error loading image %s: %s\n", "logo.png", IMG_GetError());
      exit (1);
    }

  //  printf ("Main screen images loaded\n");
  // create sub-screens.
  ns =  new numpl_screen(screen, font, graphix, starfield, aud);

  // init buttons.
  coord p0 = coord::pc (center_xcoord(logogfx), 50);
  gui_image *logoimg = new gui_image(p0, logogfx);
  coord p1 = coord::pc (center_xcoord(button_gfx[0][0]), p0.getPY() + logoimg->get_size().getPY()+10);
  gui_button *b1 = new gui_button (p1, button_gfx[0][0], button_gfx[0][1], button_gfx[0][2]);
  coord p2 = coord::pc(center_xcoord(button_gfx[1][0]), p1.getPY() + b1->get_size().getPY()+10);
  gui_button *b2 = new gui_button (p2, button_gfx[1][0], button_gfx[1][1], button_gfx[1][2]);
  coord p3 = coord::pc(center_xcoord(button_gfx[2][0]), p2.getPY() + b2->get_size().getPY()+10);
  gui_button *b3 = new gui_button (p3, button_gfx[2][0], button_gfx[2][1], button_gfx[2][2]);
  // exit from screen-button
  coord p4 = coord::pc(center_xcoord(button_gfx[3][0]), p3.getPY() + b3->get_size().getPY()+10);
  gui_button *b4 = new gui_quit_button (p4, button_gfx[3][0], button_gfx[3][1], button_gfx[3][2]);

  // set event handlers
  b1->set_act_screen(ns, numpl_screen::next_connect);
  b2->set_act_screen(ns, numpl_screen::next_host);

  // add image
  add_component(logoimg);
  // add buttons
  add_component(b1);
  add_component(b2);
  //  add_component(b3);
  delete b3;
  add_component(b4);

  focus_first_component();
}

main_screen::~main_screen()
{
  for (int i=0;i<3;i++)
    for (int j=0;j<3;j++)
      SDL_FreeSurface (button_gfx[i][j]);

  delete logogfx;
  delete ns;
}

void main_screen::activate(int signum)
{

  bool enterpushed=false;

  re_init(false);
  draw();

  aud->play_intro_music();

  SDL_Event ev;
  while (true)
    {
      draw();
      try {
	while (SDL_PollEvent(&ev))
	  {
	    if (ev.type == SDL_KEYDOWN)
	      {
		if(ev.key.keysym.sym == SDLK_DOWN)
		  focus_down();
		else if(ev.key.keysym.sym == SDLK_UP)
		  focus_up();
		else if (ev.key.keysym.sym == SDLK_ESCAPE)
		  back_one_screen();
		else
		  {
		    gui_focus_component *t = dynamic_cast<gui_focus_component *>(comps[cur_focus]);
		    if (t && (ev.key.keysym.sym == SDLK_RETURN || 
			      dynamic_cast<gui_button *>(t) == 0))
		      {
			click(t);
			enterpushed=true;
		      }
		    
		    if (t)
		      {
			t->action(ev.key.keysym.sym);
			if(enterpushed==true)
			  {
			    this->re_init(true);
			    enterpushed=false;
			  }
		      }
		  }
	      }
	  }
      } // end try
      catch (int i)
	{
	  if (i == 1) return;
	}
    }
};

void host_screen::activate(int signum)
{

  bool enterpushed=false;

  numlocal_pl = signum;

  if (numlocal_pl == 1) // 1 player
    {
      pl2img->hide();
      namebox2->hide();
    }
  else if (numlocal_pl == 2) // 2 players
    {
      pl2img->unhide();
      namebox2->unhide();
    }
  else assert(0); // shouldn't get here
  re_init(false);
  draw();

  SDL_Event ev;
  while (true)
    {
      draw();
      try {
	while (SDL_PollEvent(&ev))
	  {
	    if (ev.type == SDL_KEYDOWN)
	      {
		if(ev.key.keysym.sym == SDLK_DOWN)
		  focus_down();
		else if(ev.key.keysym.sym == SDLK_UP)
		  focus_up();
		else if (ev.key.keysym.sym == SDLK_ESCAPE)
		  back_one_screen();
		else
		  {
		    gui_focus_component *t = dynamic_cast<gui_focus_component *>(comps[cur_focus]);
		    if (t && (ev.key.keysym.sym == SDLK_RETURN &&
			      dynamic_cast<gui_button *>(t) != 0))
		      {
			click(t);
			enterpushed=true;
		      }
		    if (t)
		      {
			t->action(ev.key.keysym.sym);
			if(enterpushed==true)
			  {
			    this->re_init(true);
			    enterpushed=false;
			  }
			
		      }
		  }
	      }
	  }
      } // end try
      catch (int i)
	{
	  if (i == 1) return;
	}
    }
};

connect_screen::connect_screen(SDL_Surface *scr, TTF_Font *font, gfx *graphix, starfield_efect *se, audio *au)
  : gui_screen (scr, font, graphix, se, au)
{
  //  printf ("Loading connect_screen images:\n");
  for (int i=0;i<4;i++)
    {
      //      printf ("Loading image 'text' %s\n", imgtext_files[i]);
      SDL_Surface *s = img_load (imgtext_files[i]);
      if (s == NULL)
	{
	  fprintf (stderr, "Error loading image %s: %s\n", imgtext_files[i], IMG_GetError());
	  exit (1);
	}
      imgtext_gfx[i] = s;
    }

  for(int p=0;p<1;p++)
    {
      for (int i=0;i<3;i++)
	{
	  //	  printf ("Loading button image %s\n", button_files[p][i]);
	  SDL_Surface *s = img_load (button_files[p][i]);
	  if (s == NULL)
	    {
	      fprintf (stderr, "Error loading image %s: %s\n", button_files[p][i], IMG_GetError());
	      exit (1);
	    }
	  button_gfx[p][i] = s;
	}
    }

  coord c1 = coord::pc(SCR_WIDTH/2-imgtext_gfx[0]->w, 100);
  gui_image *t1 = new gui_image (c1, imgtext_gfx[0]);
  coord c1_2 = coord::pc (SCR_WIDTH/2-imgtext_gfx[1]->w, c1.getPY() + 30);
  pl2img = new gui_image(c1_2, imgtext_gfx[1]);
  coord c2 = coord::pc(SCR_WIDTH/2-imgtext_gfx[2]->w, c1_2.getPY() + 30);
  gui_image *t2 = new gui_image (c2, imgtext_gfx[2]);
  coord c3 = coord::pc(SCR_WIDTH/2-imgtext_gfx[3]->w, c2.getPY() + 30);
  gui_image *t3 = new gui_image (c3, imgtext_gfx[3]);

  // editbox's
  coord c4 = t1->get_position() + t1->get_size() + coord::pc(10,0);
  namebox = new gui_editbox(c4, font);
  coord c4_2 = pl2img->get_position() + pl2img->get_size() + coord::pc(10,0);
  namebox2 = new gui_editbox(c4_2, font);
  coord c5 = t2->get_position() + t2->get_size() + coord::pc(10,0);
  serverbox = new gui_editbox(c5, font);
  coord c6 = t3->get_position() + t3->get_size() + coord::pc(10,0);
  portbox = new gui_editbox(c6, font);

  // connect button
  coord c7 = coord::pc (center_xcoord(button_gfx[0][0]), c3.getPY() + t3->get_size().getPY() + 50);
  gui_button *b1 = new gui_button (c7, button_gfx[0][0], button_gfx[0][1], button_gfx[0][2]);
  b1->set_act_screen(this);
  b1->set_act_callback(start_game);

  string name ("player1");
  string name2 ("player2");
  string server("localhost");
  char buf[20];
  sprintf (buf, "%d", DEFAULT_PORT);
  string port(buf);

  namebox->set_text(name);
  namebox2->set_text(name2);
  serverbox->set_text(server);
  portbox->set_text(port);

  add_component(b1);
  add_component(t1);
  add_component(pl2img);
  add_component(t2);
  add_component(t3);
  add_component(namebox);
  add_component(namebox2);
  add_component(serverbox);
  add_component(portbox);

  focus_first_component();
}

connect_screen::~connect_screen()
{
  for (int i=0;i<1;i++)
    for (int j=0;j<3;j++)
      SDL_FreeSurface (button_gfx[i][j]);

  for (int i=0;i<4;i++)
    delete imgtext_gfx[i];
}

void connect_screen::activate(int signum)
{
  numlocal_pl = signum;

  if (numlocal_pl == 1) // 1 player
    {
      pl2img->hide();
      namebox2->hide();
    }
  else if (numlocal_pl == 2) // 2 players
    {
      pl2img->unhide();
      namebox2->unhide();
    }
  else assert(0); // shouldn't get here
  
  re_init(false);

  SDL_Event ev;
  while (true)
    {
      draw();
      try {
	while (SDL_PollEvent(&ev))
	  {
	    if (ev.type == SDL_KEYDOWN)
	      {
		if(ev.key.keysym.sym == SDLK_DOWN)
		  focus_down();
		else if(ev.key.keysym.sym == SDLK_UP)
		  focus_up();
		else if (ev.key.keysym.sym == SDLK_ESCAPE)
		  back_one_screen();
		else
		  {
		    gui_focus_component *t = dynamic_cast<gui_focus_component *>(comps[cur_focus]);
		    if (t && (ev.key.keysym.sym == SDLK_RETURN && 
			      dynamic_cast<gui_button *>(t) != 0))
		      click(t);
		    if (t)
		      {
			t->action(ev.key.keysym.sym);
			//this-re_init(true);
		      }
		  }
	      }
	  }
      } // end try
      catch (int i)
	{
	  if (i == 1) return;
	}
    }
}

void connect_screen::start_game(gui_screen *t)
{
  IPaddress server_ip;

  connect_screen *ls = dynamic_cast<connect_screen *>(t);
  string name = ls->namebox->get_text();
  string name2 = ls->namebox2->get_text();

  string names[] = {name, name2};
  
  string server = ls->serverbox->get_text();
  string port = ls->portbox->get_text();
  
  int iport = atoi(port.c_str());

  
  //  printf ("Connecting to %s:%i as %s\n", server.c_str(),iport, name.c_str());
  // check that host exists!
  if (SDLNet_ResolveHost (&server_ip, (char *) server.c_str(), iport) < 0) {
    string errs ("Error resolving server host ");
    errs += server;
    errs += ":";
    errs += port;
    errs += ".";
    ls->objs.push_back(new text_object(-1, errs));
    return;
  }

  // init client, connect to server
  game_client *client = new game_client (names, &server_ip, ls->screen, ls->font, ls->numlocal_pl, ls->graphix, ls->starfield, ls->aud);
  //  printf ("Client object created\n");

  try {
    client->connect();
  }
  catch (string &s) {
    ls->objs.push_back(new text_object(-1, s));
    return;
  }
  // start client activity
  client->activate(0);

  vector<string> er = client->get_errors();

  for (unsigned int i=0;i<er.size();i++)
    ls->objs.push_back(new text_object (-1, er[i]));

  // back to intro music
  ls->aud->play_gui_select_sound();
  ls->aud->play_intro_music();
  
  //  printf ("back from activate\n");
  delete client;
}

host_screen::host_screen(SDL_Surface *scr, TTF_Font *font, gfx *graphix, starfield_efect *se, audio *au)
  : gui_screen (scr, font, graphix, se, au)
{
  //  printf ("Loading host_screen images:\n");
  for (int i=0;i<6;i++)
    {
      //      printf ("Loading image 'text' %s\n", imgtext_files[i]);
      SDL_Surface *s = img_load (imgtext_files[i]);
      if (s == NULL)
	{
	  fprintf (stderr, "Error loading image %s: %s\n", imgtext_files[i], IMG_GetError());
	  exit (1);
	}
      imgtext_gfx[i] = s;
    }

  for(int p=0;p<1;p++)
    {
      for (int i=0;i<3;i++)
	{
	  //	  printf ("Loading button image %s\n", button_files[p][i]);
	  SDL_Surface *s = img_load (button_files[p][i]);
	  if (s == NULL)
	    {
	      fprintf (stderr, "Error loading image %s: %s\n", button_files[p][i], IMG_GetError());
	      exit (1);
	    }
	  button_gfx[p][i] = s;
	}
    }

  coord c1 = coord::pc(SCR_WIDTH/2-imgtext_gfx[0]->w, 50);
  gui_image *t1 = new gui_image (c1, imgtext_gfx[0]);
  coord c1_2 = coord::pc(SCR_WIDTH/2-imgtext_gfx[1]->w, c1.getPY() + 30);
  pl2img = new gui_image (c1_2, imgtext_gfx[1]);
  coord c2 = coord::pc(SCR_WIDTH/2-imgtext_gfx[2]->w, c1_2.getPY() + 30);
  gui_image *t2 = new gui_image (c2, imgtext_gfx[2]);
  coord c3 = coord::pc(SCR_WIDTH/2-imgtext_gfx[3]->w, c2.getPY() + 30);
  gui_image *t3 = new gui_image (c3, imgtext_gfx[3]);
  coord c4 = coord::pc(SCR_WIDTH/2-imgtext_gfx[4]->w, c3.getPY() + 30);
  gui_image *t4 = new gui_image (c4, imgtext_gfx[4]);
  coord c5 = coord::pc(SCR_WIDTH/2-imgtext_gfx[5]->w, c4.getPY() + 30);
  gui_image *t5 = new gui_image (c5, imgtext_gfx[5]);

  // editbox's
  coord c6 = t1->get_position() + t1->get_size() + coord::pc(10,0);
  namebox = new gui_editbox(c6, font);
  coord c6_2 = pl2img->get_position() + pl2img->get_size() + coord::pc(10,0);
  namebox2 = new gui_editbox(c6_2, font);
  coord c7 = t2->get_position() + t2->get_size() + coord::pc(10,0);
  portbox = new gui_editbox(c7, font);
  coord c8 = t3->get_position() + t3->get_size() + coord::pc(10,0);
  // number choosers
  num_plchooser = new gui_numchooser(c8, font, 3, 2);
  coord c9 = t4->get_position() + t4->get_size() + coord::pc(10,0);
  num_castchooser = new gui_numchooser(c9, font, 11, 4);
  coord c10 = t5->get_position() + t5->get_size() + coord::pc(10,0);
  num_winschooser = new gui_numchooser(c10, font, 5, 1);

  // connect button
  coord c11 = coord::pc(center_xcoord(button_gfx[0][0]), num_winschooser->get_position().getPY() + num_winschooser->get_size().getPY() + 50);
  gui_button *b1 = new gui_button (c11, button_gfx[0][0], button_gfx[0][1], button_gfx[0][2]);
  b1->set_act_screen(this);
  b1->set_act_callback(start_game);

  string name ("player1");
  string name2 ("player2");
  char buf[20];
  sprintf (buf, "%d", DEFAULT_PORT);
  string port(buf);

  namebox->set_text(name);
  namebox2->set_text(name2);
  portbox->set_text(port);

  num_plchooser->set_choice(DEFAULT_NUM_PLAYERS);
  num_castchooser->set_choice(DEFAULT_NUM_CASTLES);
  num_winschooser->set_choice(DEFAULT_NUM_WINS);

  add_component(b1);
  add_component(t1);
  add_component(pl2img);
  add_component(t2);
  add_component(t3);
  add_component(t4);
  add_component(t5);
  add_component(namebox);
  add_component(namebox2);
  add_component(portbox);
  add_component(num_plchooser);
  add_component(num_castchooser);
  add_component(num_winschooser);

  focus_first_component();
}

host_screen::~host_screen()
{
  for (int i=0;i<1;i++)
    for (int j=0;j<3;j++)
      SDL_FreeSurface (button_gfx[i][j]);

  for (int i=0;i<6;i++)
    delete imgtext_gfx[i];
}

void host_screen::start_game(gui_screen *t)
{
  IPaddress ip;
  IPaddress server_ip;
  game_server *serv = 0;
  game_client *client = 0;

  host_screen *ls = dynamic_cast<host_screen *>(t);
  string name = ls->namebox->get_text();
  string name2 = ls->namebox2->get_text();
  string server ("localhost");
  string port = ls->portbox->get_text();

  string names[] = {name, name2};
  int num_pl = ls->num_plchooser->get_choice();
  int num_castles = ls->num_castchooser->get_choice();
  int wins = ls->num_winschooser->get_choice();

  int iport = atoi(port.c_str());

  //  printf ("Starting server..\n");

  if (SDLNet_ResolveHost (&ip, "localhost", iport) == -1)
    {
      string errs("Error resolving host: ");
      errs += SDLNet_GetError();
      //      fprintf (stderr, "Error resolving host: %s\n", SDLNet_GetError());
      ls->objs.push_back (new text_object (-1, errs));
      return;
    }

  if (SDLNet_ResolveHost (&server_ip, NULL, iport) == -1)
    {
      string errs("Error resolving host: ");
      errs += SDLNet_GetError();
      ls->objs.push_back (new text_object (-1, errs));
      return;
    }

  // try max 10 times
  for (int i=0;i<10;i++)
    {
      ls->draw();
      try {
	if (serv) delete serv;
	serv = new game_server(server_ip, num_castles, wins);
	
	// init client, connect to server
	if (client) delete client;
	client = NULL;
	client = new game_client (names, &ip, ls->screen, ls->font, ls->numlocal_pl, ls->graphix, ls->starfield, ls->aud);
	//	printf ("Client object created\n");
	monitor_queue<msg *> *in, *out;

	client->connect_local( &in, &out);

	vector<string> nv (ls->numlocal_pl);
	for (int kk=0;kk<ls->numlocal_pl;kk++)
	  nv[kk] = names[kk];
	
	serv->add_local_connects(nv, out, in);

	// init server net
	serv->init_net();

	// start client activity
	server_numpl sn;
	sn.server = serv;
	sn.num_players = num_pl;

	SDL_Thread *th = SDL_CreateThread (serv_starter, &sn);
	if (th == 0)
	  {
	    printf ("Error starting thread!\n");
	    exit(1);
	  }
	client->activate(0);

	vector<string> es = client->get_errors();
	for (unsigned int k=0;k<es.size();k++)
	  ls->objs.push_back(new text_object(-1, es[k]));

	serv->shutdown_server();

	SDL_WaitThread(th, NULL);
	th = 0;
	i=10;
	break;
      }
      catch (string &s)
	{
	  if (client) {delete client; client = 0;}
	  //	  printf ("Error starting server: %s .. delaying and trying again.\n", ce);
	  s += " .. delaying and trying again.";
	  ls->objs.push_back (new text_object (-1, s));
	  ls->draw();
	  SDL_Delay(1000);
	}

    }

  // shutdown server
  if (serv)
    {
      //      printf ("Shutting down server..\n");      
      delete serv;
      serv = 0;
    }
  if (client)
    {
      delete client;
      client = 0;
    }

  ls->aud->play_gui_select_sound();
  ls->aud->play_intro_music();
}

int serv_starter (void *data)
{
  server_numpl *sn = static_cast<server_numpl *>(data);
  
  while (sn->server->get_mode() != game_server::ID_MODE_SHUTTING_DOWN) 
    {
      if (sn->server->no_players() == sn->num_players)
	{
	  sn->server->start_game();
	  break;
	}
      else
	SDL_Delay(100);
      // probe for disconnected players.
      sn->server->probe_disconnected_players();
    }
  return 0;
}

numpl_screen::numpl_screen(SDL_Surface *scr, TTF_Font *font, gfx *graphix, starfield_efect *se, audio *au)
  : gui_screen (scr, font, graphix, se, au)
{
  for (int i=0;i<1;i++)
    {
      //      printf ("Loading image 'text' %s\n", imgtext_files[i]);
      SDL_Surface *s = img_load (imgtext_files[i]);
      if (s == NULL)
	{
	  fprintf (stderr, "Error loading image %s: %s\n", imgtext_files[i], IMG_GetError());
	  exit (1);
	}
      imgtext_gfx[i] = s;
    }

  for(int p=0;p<2;p++)
    {
      for (int i=0;i<3;i++)
	{
	  //	  printf ("Loading button image %s\n", button_files[p][i]);
	  button_gfx[p][i] = img_load (button_files[p][i]);
	  if (button_gfx[p][i] == NULL)
	    {
	      fprintf (stderr, "Error loading image %s: %s\n", button_files[p][i], IMG_GetError());
	      exit (1);
	    }
	}
    }
  //  printf ("Number of players screen images loaded\n");
  // create sub-screens.
  ls = new connect_screen(screen, font, graphix, starfield, aud);
  hs = new host_screen(screen, font, graphix, starfield, aud);

  // init text
  coord c1 = coord::pc(center_xcoord(imgtext_gfx[0]), 100);
  gui_image *t1 = new gui_image (c1, imgtext_gfx[0]);

  // init buttons.
  coord c2 = coord::pc (center_xcoord(button_gfx[0][0]), c1.getPY() + t1->get_size().getPY() + 50);
  b1 = new gui_button (c2, button_gfx[0][0], button_gfx[0][1], button_gfx[0][2]);
  coord c3 = coord::pc (center_xcoord(button_gfx[1][0]), c2.getPY() + b1->get_size().getPY() + 20);
  b2 = new gui_button (c3, button_gfx[1][0], button_gfx[1][1], button_gfx[1][2]);

  // set event handlers - set in activate
  //  b1->set_act_screen(ls);
  //b2->set_act_screen(ls);

  // add buttons
  add_component(t1);
  add_component(b1);
  add_component(b2);

  focus_first_component();
}

numpl_screen::~numpl_screen()
{
  for (int i=0;i<2;i++)
    for (int j=0;j<3;j++)
      SDL_FreeSurface (button_gfx[i][j]);

  for (int i=0;i<1;i++)
    delete imgtext_gfx[i];

  delete ls;
  delete hs;
}

void numpl_screen::activate(int signum)
{

  bool enterpushed=false;

  if (signum == next_connect)
    {
      b1->set_act_screen(ls, 1);
      b2->set_act_screen(ls, 2);
    }
  else if (signum == next_host)
    {
      b1->set_act_screen(hs, 1);
      b2->set_act_screen(hs, 2);
    }
  else assert (0); // shouldn't get here

  re_init(false);
  draw();

  SDL_Event ev;
  while (true)
    {
      draw();
      try {
	while (SDL_PollEvent(&ev))
	  {
	    if (ev.type == SDL_KEYDOWN)
	      {
		if(ev.key.keysym.sym == SDLK_DOWN)
		  focus_down();
		else if(ev.key.keysym.sym == SDLK_UP)
		  focus_up();
		else if (ev.key.keysym.sym == SDLK_ESCAPE)
		  back_one_screen();
		else
		  {
		    gui_focus_component *t = dynamic_cast<gui_focus_component *>(comps[cur_focus]);
		    if (t && (ev.key.keysym.sym == SDLK_RETURN || 
			      dynamic_cast<gui_button *>(t) == 0))
		      {
			click(t);
			enterpushed=true;
		      }
		    if (t)
		      {
			t->action(ev.key.keysym.sym);
			if(enterpushed==true)
			  {
			    this->re_init(true);
			    enterpushed=false;
			  }
		      }
		  }
	      }
	  }
      } // end try
      catch (int i)
	{
	  if (i == 1) return;
	}
    }
};
